// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of src
//
// src is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// src is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <funtools.h>
#include <float.h>

#include <Exception/Exception.h>
#include <tracefctxx/TraceFct.h>
#include <suplibxx/str.h>

#include "FunParamWrap.h"

// we can't use this, as char* will be converted to bool, with really
// bad results. luckily we actually don't have a use for this (yet?)
//
// int FunWrapParamPut(Fun fun, char *name, int n, bool value, char *comm, int append)
// { return FunParamPutb(fun, name, n, value, comm, append); }

int FunWrapParamPut(Fun fun, const char* name, size_t idx, int value, const char* comm, int append)
{
    return FunParamPuti(fun,
                        const_cast<char*>(name),
                        static_cast<int>(idx),
                        value,
                        const_cast<char*>(comm),
                        append);
}

int FunWrapParamPut(Fun fun, const char* name, size_t idx, unsigned int value, const char* comm, int append)
{
    return FunParamPuti(fun,
                        const_cast<char*>(name),
                        static_cast<int>(idx),
                        value,
                        const_cast<char*>(comm),
                        append);
}

int FunWrapParamPut(Fun fun, const char* name, size_t idx, long unsigned int value, const char* comm, int append)
{
    return FunParamPuti(fun,
                        const_cast<char*>(name),
                        static_cast<int>(idx),
                        value,
                        const_cast<char*> (comm),
                        append);
}

int FunWrapParamPut(Fun fun, const char* name, size_t idx, double value, const char* comm, int append, int prec )
{
    return FunParamPutd(fun,
                        const_cast<char*>(name),
                        static_cast<int>(idx),
                        value,
                        prec,
                        const_cast<char*> (comm),
                        append);
}

int FunWrapParamPut(Fun fun, const char* name, size_t idx, string value, const char* comm, int append)
{ return FunParamPuts(fun,
                      const_cast<char*>(name),
                      static_cast<int>(idx),
                      (char*) value.c_str(),
                      const_cast<char*> (comm),
                      append);
}

int FunWrapParamPut(Fun fun, const char* name, size_t idx, const char* value, const char* comm, int append)
{
    return FunParamPuts(fun,
                        const_cast<char*>(name),
                        static_cast<int>(idx),
                        const_cast<char*>(value),
                        const_cast<char*> (comm),
                        append);
}

void ParamPut( Fun fun, const char* name, size_t idx, const char* val, const char* comm, int append ){
    int funret = FunWrapParamPut( fun,
                                  const_cast<char*>(name),
                                  static_cast<int>(idx),
                                  const_cast<char*>(val),
                                  const_cast<char*>(comm), append );  

  if ( ! funret ){
      TraceFct tf( "Bp2Fits::paramPut (double)" );
      throw Exception( (string) "Error writing FITS Header parameter " + string(name));
  }
}

void ParamPut( 
  Fun fun, 
  const string& name,
  size_t idx,
  double val,
  const string& comm,
  int append,
  int prec = DBL_DIG
  ){

  TraceFct tf( "Bp2Fits::paramPut (double)" );

  int funret = FunWrapParamPut( fun,
                                const_cast<char*>( name.c_str() ),
                                static_cast<int>(idx),
                                val,
                                const_cast<char*>( comm.c_str() ),
                                append, prec );  

  if ( ! funret )
    throw Exception( (string) "Error writing FITS Header parameter " + name);
}


void ParamPutType( 
  Fun fun, 
  char type,
  const string& name,
  size_t idx,
  const string& val,
  const string& comm,
  int append,
  int prec
  ) {

  try {

    switch( type ) {

    case 'A':
      ParamPut( fun, name, static_cast<int>(idx), val, comm, append );  
      break;

    case 'D':
      ParamPut( fun, name, static_cast<int>(idx), suplib::str2d(val.c_str()), comm, append, prec ); 
      break;

    case 'J':
      ParamPut( fun, name, static_cast<int>(idx), suplib::str2i(val.c_str()), comm, append ); 
      break;

    case 'V':
      ParamPut( fun, name, static_cast<int>(idx), suplib::str2ul(val.c_str()), comm, append ); 
      break;

    default:
      throw( Exception( string( "Unhandled FITS datatype `" ) + &type + "'" ) );
    }
  }
  catch ( Exception& e )
  {
    e.set_message( string( "Error converting value for FITS parameter " ) + name );
    throw( e );
  }
}

