// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of src
//
// src is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// src is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <iostream>
#include <cstdlib> // for EXIT_SUCCESS & EXIT_FAILURE
#include <suplib/file.h>
#include <Exception/Exception.h>
#include <cxcparamx/simple.h>
#include <tracefctxx/TraceFct.h>
#include "par_names.h"
#include "Bp2Fits.h"
#include "config.h"

using  std::cout;
using  std::cerr;

extern int help( void );
extern int usage( void );

int main( int argc, char* argv[] ) {

  TraceFct tf( argv[0], 0, 1 );

  try { 

    cxcparamx::simple pf( argc, argv );
  
    bool debug = pf.getb( DEBUGPAR );

    if ( pf.getb( USAGE ) ) {
      usage( );
      return EXIT_SUCCESS;

    }
    if ( pf.getb( HELP ) ) {
      help( );
      return EXIT_SUCCESS;

    }

    if ( pf.getb( "version" ) ) {
      cout << base_name( argv[0], NULL ) << "\t" << PACKAGE_VERSION << "\n";
      return EXIT_SUCCESS;
    }


  Bp2Fits bp2fits( 
      pf.getstr( INPUT ), 
      pf.getstr( OUTPUT ), 
      debug
      );

    bp2fits.trans( pf.getstr( TRANS ) );

    bp2fits.doublepass( pf.getb( DOUBLEPASS ) );
    bp2fits.longstrn(   pf.getb( LONGSTRN   ) );
    bp2fits.reserved(   pf.getb( RESERVED   ) );
    bp2fits.vectors(    pf.getb( VECTORS    ) );
    bp2fits.verbose(    pf.geti( VERBOSE    ) );

    bp2fits.add_exact_dpktfs( pf.getstr( ADD_EXACT_DPKTFS ) );
    bp2fits.add_regex_dpktfs( pf.getstr( ADD_REGEX_DPKTFS ) );
    bp2fits.del_exact_dpktfs( pf.getstr( DEL_EXACT_DPKTFS ) );
    bp2fits.del_regex_dpktfs( pf.getstr( DEL_REGEX_DPKTFS ) );

    bp2fits.writeTable( );

  } catch ( Exception& E ) {

    tf.exit( EXIT_FAILURE, E );

  } catch ( exception& e ) {
    tf.exit( EXIT_FAILURE, e.what( ) );
  }

  return EXIT_SUCCESS;

}
